"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var either_1 = require("./either");
var chai = require("chai");
var chaiAsPromised = require("chai-as-promised");
chai.use(chaiAsPromised);
var expect = chai.expect;
var anError = function () { return new Error('Something is wrong'); };
var add1 = function (n) { return n + 1; };
var removeFirstElement = function (arr) {
    if (arr.length === 0) {
        throw new Error('Array is empty');
    }
    return arr.slice(1);
};
describe('Either', function () {
    describe('isRight', function () {
        it('should return false when Left is provided', function () {
            expect(either_1.isRight(either_1.left(anError()))).to.be.false;
        });
        it('should return true when Right is provided', function () {
            expect(either_1.isRight(either_1.right('hola'))).to.be.true;
        });
        it('should return false when a non Either type is provided', function () {
            var nonEither = [1, 2, 3];
            expect(either_1.isRight(nonEither)).to.be.false;
        });
        it('should return false when null or undefined is provided', function () {
            expect(either_1.isRight(null)).to.be.false;
            expect(either_1.isRight(undefined)).to.be.false;
        });
    });
    describe('isLeft', function () {
        it('should return false when Right is provided', function () {
            expect(either_1.isLeft(either_1.right('hola'))).to.be.false;
        });
        it('should return true when Left is provided', function () {
            expect(either_1.isLeft(either_1.left(anError()))).to.be.true;
        });
        it('should return true when a non Either type is provided', function () {
            var nonEither = [1, 2, 3];
            expect(either_1.isLeft(nonEither)).to.be.true;
        });
        it('should return true when null or undefined is provided', function () {
            expect(either_1.isLeft(null)).to.be.true;
            expect(either_1.isLeft(undefined)).to.be.true;
        });
    });
    describe('withDefault', function () {
        it('should return the Right value when Right is provided', function () {
            expect(either_1.withDefault(either_1.right(6), 0)).to.equal(6);
        });
        it('should return the default value when Left is provided', function () {
            expect(either_1.withDefault(either_1.left(anError()), 0)).to.equal(0);
        });
        it('should return the default value when a non Either type is provided', function () {
            var nonEither = [1, 2, 3];
            expect(either_1.withDefault(nonEither, 2)).to.eq(2);
        });
        it('should return the default value when null or undefined is provided', function () {
            expect(either_1.withDefault(null, 1)).to.eq(1);
            expect(either_1.withDefault(undefined, 1)).to.eq(1);
        });
    });
    describe('map', function () {
        it('should return Right when mapping over Right', function () {
            var four = either_1.right(4);
            var five = either_1.map(add1, four);
            expect(either_1.isRight(five)).to.be.true;
        });
        it('should return Left when mapping over Left', function () {
            var result = either_1.map(add1, either_1.left(anError()));
            expect(either_1.isLeft(result)).to.be.true;
        });
        it('should return 5 when adding 1 to Right(4)', function () {
            var four = either_1.right(4);
            var five = either_1.map(add1, four);
            expect(either_1.isRight(five)).to.be.true;
            expect(either_1.withDefault(five, 0)).to.equal(5);
        });
        it('should remove the first element of [1,2,3] succesfully', function () {
            var list = either_1.map(removeFirstElement, either_1.right([1, 2, 3]));
            expect(either_1.isRight(list)).to.be.true;
            expect(either_1.withDefault(list, [])).to.eql([2, 3]);
        });
        it('should return a Left when mapping produces an exception', function () {
            var list = either_1.map(removeFirstElement, either_1.right([]));
            expect(either_1.isLeft(list)).to.be.true;
        });
        it('should return a Left when mapping over null, undefined or any non Either type', function () {
            expect(either_1.isLeft(either_1.map(add1, null))).to.be.true;
            expect(either_1.isLeft(either_1.map(add1, undefined))).to.be.true;
            expect(either_1.isLeft(either_1.map(add1, {}))).to.be.true;
        });
    });
    describe('andThen', function () {
        var safeRemoveFirst = function (arr) {
            try {
                return either_1.right(removeFirstElement(arr));
            }
            catch (error) {
                return either_1.left(error);
            }
        };
        it('should perform chained Right transformations', function () {
            var result = either_1.andThen(function (arr) { return safeRemoveFirst(arr); }, safeRemoveFirst(['a', 'b', 'c']));
            expect(either_1.withDefault(result, ['default val'])).to.eql(['c']);
        });
        it('should perform chained Right transformations and fail with Left', function () {
            var result = either_1.andThen(function (arr) { return either_1.andThen(function (arr2) { return safeRemoveFirst(arr2); }, safeRemoveFirst(arr)); }, safeRemoveFirst(['a', 'b']));
            expect(either_1.withDefault(result, ['default val'])).to.eql(['default val']);
        });
        it('should return left throw when chaining over null, undefined or any non Either type', function () {
            expect(either_1.isLeft(either_1.andThen(function (a) { return either_1.right(a); }, null))).to.be.true;
            expect(either_1.isLeft(either_1.andThen(function (a) { return either_1.right(a); }, undefined))).to.be.true;
            expect(either_1.isLeft(either_1.andThen(function (a) { return either_1.right(a); }, {}))).to.be.true;
        });
    });
    describe('caseOf', function () {
        it('should Launch 5 missiles', function () {
            return expect(either_1.caseOf({
                Left: function (err) { return "Error: " + err.message; },
                Right: function (n) { return "Launch " + n + " missiles"; }
            }, either_1.right('5'))).to.eventually.equal('Launch 5 missiles');
        });
        it('should error', function () {
            return expect(either_1.caseOf({
                Left: function (err) { return "Error: " + err.message; },
                Right: function (n) { return "Launch " + n + " missiles"; }
            }, either_1.left(anError()))).to.be.rejectedWith('Error: Something is wrong');
        });
    });
    describe('tryCatch', function () {
        var shouldFail;
        var fn = function () {
            if (shouldFail) {
                throw anError();
            }
            return 'Ok';
        };
        it('should convert a failing function into a Left', function () {
            shouldFail = true;
            return expect(either_1.caseOf({
                Left: function (err) { return "Error: " + err.message; },
                Right: function (n) { return "Launch " + n + " missiles"; }
            }, either_1.tryCatch(fn, function (err) { return err; }))).to.be.rejectedWith('Error: Something is wrong');
        });
        it('should convert a successful function into a Right', function () {
            shouldFail = false;
            expect(either_1.tryCatch(fn, function (err) { return err; })).to.deep.equal(either_1.right('Ok'));
        });
    });
    describe('examples', function () {
        var bandsJsonWithContent = {
            'bands.json': "\n        [\n          {\"artist\": \"Clark\", \"bio\": \"Clark bio...\"},\n          {\"artist\": \"Plaid\", \"bio\": \"Plaid bio...\"}\n        ]\n        "
        };
        var bandsJsonWithoutContent = {
            'bands.json': ''
        };
        var generateExample = function (filenameToRead, folder) {
            var readFile = function (filename) {
                if (folder.hasOwnProperty(filename)) {
                    return either_1.right(folder[filename]);
                }
                return either_1.left(new Error("File " + filename + " doesn't exist"));
            };
            var bandsJson = readFile(filenameToRead);
            var bands = either_1.map(function (json) { return JSON.parse(json); }, bandsJson);
            var bandNames = either_1.map(function (bands) { return bands.map(function (band) { return band.artist; }); }, bands);
            return bandNames;
        };
        it('should be all right', function () {
            var bandNames = generateExample('bands.json', bandsJsonWithContent);
            return expect(either_1.caseOf({
                Left: function (err) { return err.message; },
                Right: function (names) { return names; }
            }, bandNames)).to.eventually.deep.equal(['Clark', 'Plaid']);
        });
        it("should fail becasue File xyz doesn't exist", function () {
            var bandNames = generateExample('non-existing.json', bandsJsonWithContent);
            return expect(either_1.caseOf({
                Left: function (err) { return err.message; },
                Right: function (names) { return names; }
            }, bandNames)).to.be.rejectedWith("File non-existing.json doesn't exist");
        });
        it("should fail becasue File content is not valid Json", function () {
            var bandNames = generateExample('bands.json', bandsJsonWithoutContent);
            return expect(either_1.caseOf({
                Left: function (err) { return err.message; },
                Right: function (names) { return names; }
            }, bandNames)).to.be.rejectedWith("Unexpected end of JSON input");
        });
        it('should be a good tryCatch example', function () {
            var res = either_1.tryCatch(function () { return JSON.parse(''); }, function (err) { return err; });
            return expect(either_1.caseOf({
                Left: function (err) { return err.message; },
                Right: function (result) { return result; }
            }, res)).to.be.rejectedWith("Unexpected end of JSON input");
        });
    });
});
